#
# spec file for package jctools
#
# Copyright (c) 2023 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global srcname JCTools
Name:           jctools
Version:        3.3.0
Release:        3.1
Summary:        Java Concurrency Tools for the JVM
License:        Apache-2.0
URL:            https://github.com/JCTools/JCTools
Source0:        %{url}/archive/v%{version}/%{srcname}-%{version}.tar.gz
Patch0:         imports.patch
BuildRequires:  fdupes
BuildRequires:  java-devel >= 1.8
BuildRequires:  maven-local
BuildRequires:  mvn(com.github.javaparser:javaparser-core) >= 3.14.16
BuildRequires:  mvn(org.apache.felix:maven-bundle-plugin)
BuildRequires:  mvn(org.codehaus.mojo:exec-maven-plugin)
BuildRequires:  mvn(org.ow2.asm:asm-all)
BuildArch:      noarch

%description
This project aims to offer some concurrent data structures
currently missing from the JDK:

* SPSC/MPSC/SPMC/MPMC Bounded lock free queues
* SPSC/MPSC Unbounded lock free queues
* Alternative interfaces for queues
* Offheap concurrent ring buffer for ITC/IPC purposes
* Single Writer Map/Set implementations
* Low contention stats counters
* Executor

%package channels
Summary:        JCTools Channel implementations

%description channels
Channel implementations for the
Java Concurrency Tools Library.

%package experimental
Summary:        JCTools Experimental implementations

%description experimental
Experimental implementations for the
Java Concurrency Tools Library.

%package javadoc
Summary:        Javadoc for %{name}

%description javadoc
This package contains javadoc for %{name}.

%prep
%setup -q -n %{srcname}-%{version}
%patch0 -p1

# set correct version in all pom.xml files
%pom_xpath_set pom:project/pom:version %{version}
%pom_xpath_set pom:parent/pom:version %{version} jctools-{build,core,channels,experimental}

# remove plugins unnecessary for RPM builds
%pom_remove_plugin :coveralls-maven-plugin jctools-core
%pom_remove_plugin :jacoco-maven-plugin jctools-core
%pom_remove_plugin :maven-enforcer-plugin
%pom_remove_plugin :maven-source-plugin jctools-core
%pom_remove_plugin :maven-javadoc-plugin jctools-core

# disable unused modules with unavailable dependencies
%pom_disable_module jctools-benchmarks
%pom_disable_module jctools-concurrency-test

%pom_xpath_set "pom:project/pom:properties/pom:java.version" "1.8"
%pom_xpath_set "pom:project/pom:properties/pom:java.test.version" "1.8"

# do not install internal build tools
%{mvn_package} :jctools-build __noinstall

# do not install unused parent POM
%{mvn_package} :jctools-parent __noinstall

%build
%{mvn_build} -sf -- \
    -Dproject.build.outputTimestamp=$(date -u -d @${SOURCE_DATE_EPOCH:-$(date +%%s)} +%%Y-%%m-%%dT%%H:%%M:%%SZ) \
    -Dsource=8

%install
%mvn_install
%fdupes -s %{buildroot}%{_javadocdir}

%files -f .mfiles-jctools-core
%doc README.md
%license LICENSE

%files channels -f .mfiles-jctools-channels

%files experimental -f .mfiles-jctools-experimental

%files javadoc -f .mfiles-javadoc
%license LICENSE

%changelog
* Sat Sep  9 2023 Fridrich Strba <fstrba@suse.com>
- Reproducible builds: use SOURCE_DATE_EPOCH for timestamp
* Wed Mar 23 2022 Fridrich Strba <fstrba@suse.com>
- Build with java source and target levels 8
* Thu Mar 10 2022 Fridrich Strba <fstrba@suse.com>
- Upgrade to 3.3.0
  * API Changes:
    + Removed MpscLinkedQueue7 and MpscLinkedQueue8 and consolidated
    into parent. This removes the need for the builder method on
    MpscLinkedQueue.
    + Deprecated QueueFactory and spec package classes. These are
    not used by any users AFAICT and are only used for testing
    internally.
    + Removed some internal classes and reduced visibility of
    internal utilities where practical. The @InternalAPI
    tagging annotation is also used more extensively to discourage
    dependency.
    + #226: XADD unbounded mpsc/mpmc queue: highly scalable linked
    array queues
    + New blocking consumer MPSC
  * Bug fixes:
    + #334 NBHMLong iterators do not remove NO_KEY
    + #335 NBHM/Long/Identity iterators only removes keys if values
    have not changed. Now follow JDK convention.
    + #336 MpscBlockingConsumerArrayQueue: fix race writing to
    blocked field
    + #339 'fill' wakeup call can spin forever (or until a consumer
    is blocked again) on MpscBlockingConsumerArrayQueue
    + #319 Relying on test cases provided by #328 we apply the fix
    used for #205 to 'NonBlockingHashMapLong' and
    'NonBlockingIdentityHashMap' to provide 'getAndSet' like
    semantics for put and remove.
    + #330 Fix a 'NonBlockingIdentityHashMap' 'replace' bug. During
    this fix some further work was done to bring the code closer
    in line to the current state of 'NonBlockingHashMap'
    + Use byte fields for padding (avoid upcoming false sharing
    problem in JDK 15+ where field ordering has changed)
    + #289 Add Automatic-Module-Name header to MANIFEST.MF
    + #292 Fix inconsistent isEmpty/poll/peek/offer dynamics for
    SpscLinkedQueue
    + Fixed potential for negative queue size for indexed queues and
    similar issue with isEmpty
    + #296 Fix peek/relaxedPeek race with poll/offer in MC queues
    + #297 Fix inconsistent size of FF based queues causing
    potential size() > capacity()
    + #316 Fix MpscBlockingConsumerArrayQueue::poll(TimeUnit,timeout)
    + #310 Fix MpmcUnboundedXaddArrayQueue::peek/relaxedPeek
    can load "future" elements
    + #209: On Arm7, non-volatile long can have unaligned address
    leading to error
    + #216: Size of SpscGrowableArrayQueue can exceeds max capacity
    + #241: Protect the producer index in case of OutOfMemoryError
    + #244: Long NBHM AssertionError when replacing missing key
    + Fix argument checks on fill/drain methods
    + Fix LGTM warning, potential int overflow bug
  * Enhancements:
    + #326 Xadd queues consumers can help producers
    + #323 Update to latest JCStress (thanks @shipilev )
  * Improvements:
    + Don't mark generated linked atomic queues as final
    + #211: Implement batching methods on MpmcArrayQueue
    + #228: Iterator for MpscArrayQueue and MpscUnboundedArrayQueue
    + Iterator support also available for the *ArrayQueue classes
    + #208: MpscLinkedAtomicQueue can be made not final
    + #237: Add scale to exception message to help debug
    netty/netty#8916
  * New features:
    + #340 MpscBlockingConsumerArrayQueue
    + After long incubation and following a user request (see #321),
    we move counters into core
    + Merging some experimental utils and a #264 we add a
    'PaddedAtomicLong'
    + #314 MpscBlockingConsumerArrayQueue::offerIfBelowThreshold is
    added
- Added patch:
  * imports.patch
    + Fix bug in adding of imports that broke build with
    javaparser >= 2.15.0
* Tue Oct  1 2019 Fridrich Strba <fstrba@suse.com>
- Initial packaging of jctools 2.1.2
